unit Drives;

{ see TEST program below !! }

{ Unit Drives, written by Salvatore Besso   }
{ mc8505@mclink.it                          }

{ This unit is freeware and is donated to   }
{ the SWAG archival group.                  }

{ Finally, a Drives unit that correctly     }
{ works in both real and protected mode,    }
{ in a Windows 95 DOS box, and that doesn't }
{ require a media to be present in the      }
{ removable drive.                          }

{ This unit is still not able to correctly  }
{ recognize Iomega Zip drives in a Windows  }
{ 95 DOS box for now (they are recognized   }
{ as removable media). As soon as new       }
{ informations will be available from the   }
{ interrupt list of Ralph Brown, the unit   }
{ will be modified. Actually informations   }
{ about Iomega interrupt are very scarce.   }

{ A new Dpmi unit is beyond the end of this }
{ unit                                      }

{ Test program is beyond the end of the     }
{ Drives and Dpmi units                     }

{ If you have any feedback, feel free to    }
{ e-mail me                                 }

interface

uses
  {$IFDEF DPMI}
  Dpmi,
  {$ENDIF}
  Dos;

const

  { dtXXXX constants - Drive Type }

  dtInvalid   = $0;
  dtUnknown   = $1;

  { Floppy disk }

  dt8Single   = $2;
  dt8Double   = $4;
  dt360       = $8;
  dt1200      = $10;
  dt720       = $20;
  dt1440      = $40;
  dt2880      = $80;
  dtAnyFloppy = $FE;

  { Other media }

  dtTape      = $100;
  dtFloptical = $200;
  dtRamDisk   = $400;
  dtCdRom     = $800;
  dtIomegaZip = $1000;

  dtHardDisk  = $80000;

  { Other attributes }

  dtRemovable = $100000;
  dtRemote    = $200000;

type

  PParamBlock = ^TParamBlock;
  TParamBlock = record
    SpecialFunctions: Byte;     { Special functions }
    DeviceType      : Byte;     { Device type }
    DeviceAttributes: Word;     { Device attributes }
    MaxCylinders    : Word;     { Number of cylinders }
    MediaType       : Byte;     { Media type }
    { Beginning of BIOS parameter block (BPB) }
    BytesPerSector  : Word;     { Bytes per sector }
    SectPerCluster  : Byte;     { Sectors per cluster }
    ReservedSectors : Word;     { Number of reserved sectors }
    NumberFats      : Byte;     { Number of FATs }
    RootDirEntries  : Word;     { Number of root-directory entries }
    TotalSectors    : Word;     { Total number of sectors }
    MediaDescriptor : Byte;     { Media descriptor }
    SectorsPerFat   : Word;     { Number of sectors per FAT }
    SectorsPerTrack : Word;     { Number of sectors per track }
    NumberHeads     : Word;     { Number of heads }
    HiddenSectors   : LongInt;  { Number of hidden sectors }
    HugeSectors     : LongInt   { Number of sectors if TotalSectors = 0 }
    { End of BIOS parameter block (BPB) }
  end;

  PtrRec = record   { replicated from OBJECTS.PAS to avoid using the unit }
    Ofs,Seg: Word
  end;

  DriveLetters = 'A'..'Z';
  DriveSet     = Set of DriveLetters;

{ returns all available drives in a DriveSet type variable }

procedure GetDrives (var Drive: DriveSet);

{ returns drive type }

function GetDriveType (Drive: Char): LongInt;

implementation

procedure GetDrives (var Drive: DriveSet);

var
  DriveName: array[1..2] of Char;
  FCB      : array[0..43] of Char;
  Dr       : LongInt;

begin
  asm
        PUSH    SI
        PUSH    DI
        PUSH    ES
        PUSH    DS
        MOV     SI,SS     { Stack points to local variables }
        MOV     DS,SI     { also DS ... }
        PUSH    DS
        POP     ES        { ...and ES }
        MOV     BYTE PTR [DriveName],'A'
        MOV     BYTE PTR [DriveName + 1],':'
        MOV     WORD PTR [Dr],0
        MOV     WORD PTR [Dr + 2],0
        MOV     DX,1
        XOR     CX,CX
  @@1:  LEA     SI,DriveName
        LEA     DI,FCB
        MOV     AX,290EH  { Function 29H - Parse Filename - AL = options }
        INT     21H
        CMP     AL,0FFH
        JE      @@2
        PUSH    DX
        PUSH    CX
        MOV     AX,4409H  { SUBST drives are ignored }
        MOV     BL,BYTE PTR [DriveName]
        SUB     BL,'@'
        INT     21H
        JC      @@2
        TEST    DH,10000000B
        POP     CX
        POP     DX
        JNZ     @@2
        OR      WORD PTR [Dr],DX
        OR      WORD PTR [Dr + 2],CX
  @@2:  SHL     DX,1
        RCL     CX,1
        INC     BYTE PTR [DriveName]
        CMP     BYTE PTR [DriveName],'Z'
        JBE     @@1
        SHL     WORD PTR [Dr],1
        RCL     WORD PTR [Dr + 2],1
        POP     DS
        POP     ES
        POP     DI
        POP     SI
  end;
  Drive := DriveSet (Dr)
end;

function GetDriveType (Drive: Char): LongInt;

var
  DPB    : PParamBlock;
  SegInfo: Word;
  Regs   : Registers;
  Temp   : Byte;
  Result : LongInt;
  {$IFDEF DPMI}
  Size   : LongInt;
  {$ENDIF}

function GetDevParms (Drive: Char; var DPB: PParamBlock; Segm: Word): Boolean;

var
  Regs: Registers;

begin
  GetDevParms := False;
  FillChar (Regs,SizeOf (Registers),0);
  Regs.AX := $440D;
  Regs.BL := Byte (Drive) - 64;
  Regs.CH := $08;                       { category: disk drive }
  Regs.CL := $60;                       { device parameters    }
  {$IFNDEF DPMI}
  Regs.DS := PtrRec (DPB).Seg;
  Regs.DX := PtrRec (DPB).Ofs;
  MsDos (Regs);
  {$ELSE}
  Regs.DS := Segm;
  Regs.DX := 0;
  if NOT DpmiMsDos (Regs) then Exit;
  {$ENDIF}
  GetDevParms := Regs.Flags and fCarry = 0
end;

function IsDriveRemote (Drive: Char): Boolean; assembler;

asm
        MOV     AX,4409H  { IOCTL - Check if block device remote }
        MOV     BL,Drive  { BL = drive                           }
        SUB     BL,'@'    { 1 = A:, 2 = B:, etc...               }
        INT     21H
        XOR     AX,AX
        JC      @@1
        AND     DH,00010000B
        JZ      @@1
        INC     AX
  @@1:
end;

function IsCDRomDrive (Drive: Char): Boolean; assembler;

asm
        MOV     AX,150BH  { MSCDEX.EXE installation test }
        XOR     CH,CH     { CX = drive                   }
        MOV     CL,Drive
        SUB     CL,'A'    { 0 = A:, 1 = B:, etc...       }
        INT     2FH
        PUSH    AX
        POP     CX
        XOR     AX,AX
        JCXZ    @@1
        TEST    BX,0ADADH
        JZ      @@1
        INC     AX
  @@1:
end;

function IsIomegaZip: Boolean;

var
  Regs  : Registers;
  Result: Boolean;

begin
  { Find first GUEST.EXE... }
  FillChar (Regs,SizeOf (Registers),0);
  Regs.AX := $5700;                      { GUEST.EXE installation test }
  Regs.BX := $0201;                      { Iomega ID ???               }
  Regs.DX := $496F;                      { 'Io'                        }
  {$IFNDEF DPMI}
  Intr ($2F,Regs);
  {$ELSE}
  if NOT DpmiIntr ($2F,Regs) then Exit;
  {$ENDIF}
  Result := Regs.AL = $FF;
  if NOT Result then
  begin
    { ...GUEST.EXE not found: Find GUEST95.EXE...  }
    { Interrupt informations for GUEST95.EXE still }
    { not available                                }
  end;
  IsIomegaZip := Result
end;

begin { GetDriveType }
  GetDriveType := dtInvalid;
  {$IFNDEF DPMI}
  New (DPB);
  SegInfo := 0;
  {$ELSE}
  Size := SizeOf (TParamBlock);
  if NOT DpmiGetMem (Pointer (DPB),SegInfo,Size) then Exit;
  {$ENDIF}
  FillChar (DPB^,SizeOf (TParamBlock),0);
  FillChar (Regs,SizeOf (Regs),0);
  Regs.AX := $4408;                     { removable media ? }
  Regs.BL := Byte (Drive) - 64;
  {$IFNDEF DPMI}
  MsDos (Regs);
  {$ELSE}
  if NOT DpmiMsDos (Regs) then
  begin
    DpmiFreeMem (Pointer (DPB));
    Exit
  end;
  {$ENDIF}
  Temp := 0;
  if Regs.Flags and fCarry <> 0 then  { error, check error code in AX }
  begin
    { Driver does NOT support this call, so guess as a hard disk }
    if Regs.AX = 1 then Temp := 3
  end
  else begin
    if Regs.AX = 0 then
      Temp := 2          { removable media, floppy, WORM, Floptical, ZIP }
    else Temp := 3       { or hard disk, ramdisk or CD-ROM               }
  end;
  Result := dtInvalid;
  case Temp of
    { Removable }
    2: if GetDevParms (Drive,DPB,SegInfo) then
    begin
      case DPB^.DeviceType of
        0: Result := dt360;
        1: Result := dt1200;
        2: Result := dt720;
        3: Result := dt8Single;
        4: Result := dt8Double;
        5: if IsIomegaZip then Result := dtIomegaZip else Result := dtHardDisk;
        6: Result := dtTape;
        7: Result := dt1440;
        8: Result := dtFloptical;
        9: begin
          if (DPB^.MaxCylinders = 80) and (DPB^.NumberHeads = 2) then
            Result := dt2880
          else if IsIomegaZip then
            Result := dtIomegaZip
          else Result := dtUnknown
        end
        else Result := dtUnknown
      end;
      if Result > dtUnknown then Result := Result or dtRemovable
    end;
    { Fixed }
    3: if GetDevParms (Drive,DPB,SegInfo) then
      if DPB^.DeviceType = 5 then
        Result := dtHardDisk
      else Result := dtUnknown
    else Result := dtRamDisk
  end;
  if IsDriveRemote (Drive) then
    if IsCDRomDrive (Drive) then
      Result := dtCdRom or dtRemovable
    else Result := Result or dtRemote;
  {$IFNDEF DPMI}
  Dispose (DPB);
  {$ELSE}
  if NOT DpmiFreeMem (Pointer (DPB)) then Exit;
  {$ENDIF}
  GetDriveType := Result
end;

end.

