const ProgName   :string[80] = 'VESA driver interface by P'#$0D#$0A'$';
          ProgVersion:word = $0102;  { programmed for VESA version 1.2 }
          MaxVertical =      480*2;       { max. vertical lines you need }
                                                                          { double if 2 pages required }

          VesaSign   :longint = $41534556; { = 'VESA' }
          StatusOk   :WORD    = $004f;     { default ok response after int 10 }
                                                                                          {.........!.........!.........!}
          VesaErrors :ARRAY[0..5] OF STRING[30] =('Vesa init ok.                 ',
                                                                                          '?Error getting vesainfo.      ',
                                                                                          '?Vesa not detected.           ',
                                                                                          '?Vesa not detected.           ',
                                                                                          '?Error getting vesamodeinfo.  ',
                                                                                          '?Error opening vesamode.      ');
          MaxBanks = 256;    { maximum bank switch }

type
 TVerticalPosInfo = ARRAY[0..MaxVertical-1] OF RECORD { extra help for finding}
                                                         Address :Word;             { memory addres }
                                                         Bank:Byte;             { memory bank }
                                                         reserved:byte;
                                                         end;

 TVesaInfoBlock = record
                VESASignature      : longint; {'VESA' check as number}
                VESAVersion        : word;    { version hi.lo }
                OEMStringPtr       : pointer; { pointer to OEM string }
                Capabilities       : longint; { Capabilities field }
                VideoModePtr       : pointer; { pointer to supported modes }
                TotalMemory        : word;    { x64k available memory }
                reserved           : array[1..236] of byte; {unused yet}
                end;

 TVesaModeInfoBlock = record
                ModeAttributes     : word;    { mode attributes                    }
                WinAAttributes     : byte;    { window A attributes                }
                WinBAttributes     : byte;    { window B attributes                }
                WinGranularity     : word;    { window granularity                 }
                WinSize            : word;    { window size                        }
                WinASegment        : word;    { window A start segment             }
                WinBSegment        : word;    { window B start segment             }
                WinFuncPtr         : pointer; { pointer to windor function         }
                BytesPerScanLine   : word;    { bytes per scan line                }
                XResolution        : word;    { horizontal resolution              }
                YResolution        : word;    { vertical resolution                }
                XCharSize          : byte;    { character cell width               }
                YCharSize          : byte;    { character cell height              }
                NumberOfPlanes     : byte;    { number of memory planes            }
                BitsPerPixel       : byte;    { bits per pixel                     }
                NumberOfBanks      : byte;    { number of banks                    }
                MemoryModel        : byte;    { memory model type                  }
                BankSize           : byte;    { bank size in kb                    }
                NumberOfImagePages : byte;    { number of images                   }
                Reserved           : byte;    { reserved for page function         }
                RedMaskSize        : byte;    { size of direct color red mask      }
                RedFieldPosition   : byte;    { bit position of LSB of red mask    }
                GreenMaskSize      : byte;    { size of direct color green mask    }
                GreenFieldPosition : byte;    { bit position of LSB of green mask  }
                BlueMaskSize       : byte;    { size of direct color blue mask     }
                BlueFieldPosition  : byte;    { bit position of LSB of blue mask   }
                RsvdMaskSize       : byte;    { size of direct color reserved mask }
                DirectColorModeInfo: byte;    { Direct Color mode attributes       }
                Reserved2          : array[1..216] of byte; { remainder            }
           end;
var
 VesaInfo       : TVesaInfoBlock;     { get your own info here }
 VesaModeInfo   : TVesaModeInfoBlock; { get your own info here }
 VerticalPosInfo: TVerticalPosInfo;   { get your own info here }
 BankTable      : ARRAY[0..MaxBanks] OF Word;

 CurrentBank,
 WritePage,
 VisualPage       : word;
 WinSizeBytes     : word;  { winsize in bytes..   0 = 64k }


procedure PutPixelVESA(X,Y:Integer; C:byte);assembler;
asm
                 mov     ax,Y
                 add     ax,WritePage

                 mov     di,X
                 mov     dx,VesaModeInfo.BytesPerScanLine
                 mul     dx
                 add     di,ax
                 adc     dx,0
                 cmp     dx,CurrentBank         { out of current window boundary? }
                 je      @@1                    { no }
{==== bankswitch ====}
           MOV     BX,DX
                 MOV     CurrentBank,BX

           ADD     BX, BX
           ADD     BX, OFFSET BankTable
           mov     DX, DS:[BX]
           mov     AX, $4F05
           xor     BX, BX
           int     10h
@@1:
           mov     Al ,c
           mov     es,SegA000
           mov     es:[di], al
end;
function GetPixelVESA(X,Y:Integer):byte;
var getp:byte;
begin
 asm
                 mov     ax,Y
                 add     ax,WritePage

                 mov     di,X
                 mov     dx,VesaModeInfo.BytesPerScanLine
                 mul     dx
                 add     di,ax
                 adc     dx,0
                 cmp     dx,CurrentBank         { out of current window boundary? }
                 je      @@1                    { no }
 {==== bankswitch ====}
           MOV     BX,DX
                 MOV     CurrentBank,BX

           ADD     BX, BX
           ADD     BX, OFFSET BankTable
           mov     DX, DS:[BX]
           mov     AX, $4F05
           xor     BX, BX
           int     10h
 @@1:
           mov     es,SegA000
           mov     al,es:[di]
           mov     GetP, al
 end;
GetPixelVESA:=getp;
end;
function setVESAmode(mode:word):boolean;
var i:word;
begin
          @putpixel:=@putpixelVESA;
          @getpixel:=@getpixelVESA;
          case mode of
           m800x600:begin maxx:=799;maxy:=599;end;
           m1024x768:begin maxx:=1023;maxy:=767;end;
          end;
          colors:=16;
asm
   xor  si,si        { Error step value }
   inc  si           { start with one }

   mov  ax,ds
   mov  es,ax
   mov  ax,$4f00;    { return SuperVGA information }
   mov  di, OFFSET VesaInfo; { into buffer VesaInfo }
   int  $10;

   cmp  ax,StatusOk; { Status okay ? }
{1}   jnz @ErrorInit;    { no, then halt init }

   inc  si            { next check }
   mov  ax, WORD PTR VesaInfo.VESASignature;  { check first two chars }
   cmp  ax, WORD PTR VesaSign
{2}   jnz @ErrorInit

   inc  si            { next check }
   mov  ax, WORD PTR VesaInfo.VESASignature+2;  { check next two chars }
   cmp  ax, WORD PTR VesaSign+2
{3}   jnz @ErrorInit

   { primary test for VESA done, try to get videomode info }
   inc  si            { next check }
   mov  ax, $4f01               { return videomode information }
   mov  cx, Mode         { requested mode }
   mov  di, OFFSET VesaModeInfo { into buffer VesaModeInfo }
   int  $10;

   cmp  ax, StatusOk;           { did this work out good? }
{4}   jnz @ErrorInit

   { start opening videomode }
   inc  si            { next check }
   mov  ax, $4f02;
   mov  bh, 0;                { must clear screen }
   xor  bh, $01;                { invert bit }
   shl  bh, 7;                  { move bit to d15 }
   xor  bl, bl;                 { clear lower byte }
   or   bx, Mode;        { combine with videomode }
   int  $10;

   cmp  ax, StatusOk            { did this work out good? }
{5}   jnz @ErrorInit

   { build vertical pos info block }
   inc  si               { next check }
   mov  cx, MaxVertical  { how many times }
   mov  di, OFFSET VerticalPosInfo;

   xor  bx, bx;                 { startbank=0 }
   xor  ax, ax;                 { address  =0 }
   mov  dx, WORD PTR VesaModeInfo.WinSize;
   mov  WinSizeBytes, dx
   xchg dl, dh                  { multiply this with 1024 }
   shl  dh, 2
(*    mov  dx, $1000 *)

   { set info }
@VPosLoop:
   mov  ds:[di+0],ax            { place addres in vpos infoblock}
   mov  ds:[di+2],bl            { place bank in vpos infoblock }
   mov  ds:[di+3],bh            { place bank in vpos infoblock }

   add  di,4                    { add to next pos. in array}
   add  ax, WORD PTR VesaModeInfo.XResolution { increment address}
   jc   @IncBank                { in case 64k buffer carry is gen. here! }

   cmp  ax, dx                  { check if bank is passed}
   jb  @DoLoop;

   sub  ax, dx                  { decrement ax with size to start again 4k-buf}
@IncBank:
   inc  bx;

@DoLoop:
   Loop @VPosLoop;

   { built bankswitchtable }

   mov  di, OFFSET BankTable
   xor  bx, bx        { start at bank 0 }

@BankLoop:
   mov   ax,bx        { get banknr }

   mov   cx, 64
   mul   cx
        mov   cx, WORD PTR VesaModeInfo.WinGranularity
   jcxz    @@be
   div   cx
@@be:
   mov   ds:[di],ax   { save value }
   inc   di
   inc   di

   inc   bx
   cmp   bx,MaxBanks
   jb    @BankLoop

   { buffer has been built for easy use! }
   XOR  CX,CX
   mov  CurrentBank, CX   { clear current bank to 0 first time}
   mov  WritePage,   CX   { write to page 0 }
   mov  VisualPage,  CX   { visual page = 0 }
   xor  bx,bx;            { perpare for int.}
   mov  dx,cx;            { get banknr from cx }
   mov  ax,$4f05;         { irq init }
   int $10;

(*    jmp  @okinit; *)

{ all ok part }
@OkInit:
   XOR  AX,AX
@ErrorInit:
@End:
end;
end;

